#  Copyright (c) 2019-2023  Marc van der Sluys - marc.vandersluys.nl
#   
#  This file is part of the HistAstro Python package,
#  see: http://astro.ru.nl/~sluys/HistAstro/
#   
#  This is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
#  
#  This software is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
#  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#  
#  You should have received a copy of the GNU General Public License along with this code.  If not, see
#  <http://www.gnu.org/licenses/>.


"""Plot-related functions for HistAstro."""


def mag2size(Mlim, mag, scale=1):
    """
    Convert magnitudes (mag) to disc area ('size') for plotting with pyplot.scatter(), given a magnitude limit Mlim.
    
    Args:
      Mlim (float):   Magnitude limit.
      mag (float):    Magnitude (Numpy array).
      scale (float):  Scale factor (default value = 1).
    
    Returns:
      float:  Area sizes for the stellar discs (Numpy array)
    
    """
    
    size = scale * 30*(0.5 + (Mlim-mag)/3.0)**2     # Disc area as a function of magnitude
    
    return size




#  Copyright (c) 2019-2023  Marc van der Sluys - marc.vandersluys.nl
#   
#  This file is part of the HistAstro Python package,
#  see: http://astro.ru.nl/~sluys/HistAstro/
#   
#  This is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
#  
#  This software is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
#  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#  
#  You should have received a copy of the GNU General Public License along with this code.  If not, see
#  <http://www.gnu.org/licenses/>.


"""Planet functions for HistAstro."""

import math as m
import numpy.core as np
import histastro.datetime as dt

pi2 = m.pi*2


def readVSOP(dataDir, pl):
    """
    Read the periodic terms for a heliocentric ecliptical planet position from a VSOP87D.* file.
    
    Args:
      dataDir (str):  Directory where the VSOP87D.* files are located.
      pl (int):       Planet ID: 1-8 = Mercury - Neptune.
    
    Returns:
      tuple (float,float,float):  Tuple containing (lonTerms, latTerms, radTerms):
    
      - lonTerms (float):  Numpy array containing VSOP87D periodic terms for heliocentric ecliptical
                           longitude.
      - latTerms (float):  Numpy array containing VSOP87D periodic terms for heliocentric ecliptical latitude.
      - radTerms (float):  Numpy array containing VSOP87D periodic terms for heliocentric distance.
    
    References:
      - [Bretagnon & Francou, A&A 202, 309 (1988)](https://ui.adsabs.harvard.edu/abs/1988A%26A...202..309B).
      - [VSOP87D.* data files](http://cdsarc.u-strasbg.fr/viz-bin/Cat?cat=VI/81): click on FTP, download VSOP87D.*.
    
    """
    
    exts = ['mer','ven','ear','mar','jup','sat','ura','nep']
    fileName = dataDir+'/VSOP87D.'+exts[pl-1]
    inFile = open(fileName,'r')
    
    import fortranformat as ff
    formatHeader = ff.FortranRecordReader('(40x,I3, 16x,I1,I8)')         # Block header format
    formatBody   = ff.FortranRecordReader('(79x,F18.11,F14.11,F20.11)')  # Block body format
    
    lonTerms=[]; latTerms=[]; radTerms=[]
    
    for iBlock in range(3*6):  # 3 variables (l,b,r), up to 6 powers (0-5)
        line = inFile.readline()
        var,power,nTerm = formatHeader.read(line)
        # print(var,power,nTerm)
        if line == '': break  # EoF
        
        for iLine in range(nTerm):
            line = inFile.readline()
            a,b,c = formatBody.read(line)
            # print(iLine, var,power, a,b,c)
            
            if var == 1: lonTerms.append([power, a,b,c])  # var=1: ecliptic longitude
            if var == 2: latTerms.append([power, a,b,c])  # var=2: ecliptic latitude
            if var == 3: radTerms.append([power, a,b,c])  # var=3: radial distance
            
    return lonTerms,latTerms,radTerms


def computeLBR(jde, lonTerms,latTerms,radTerms):
    """
    Compute the heliocentric ecliptical coordinates for a planet from its VSOP87D periodic terms.
    
    Args:
      jde (float):       Julian Day in dynamical time, i.e., corrected for Delta T (days).
      lonTerms (float):  Numpy array containing VSOP87D periodic terms for heliocentric ecliptical
                         longitude.
      latTerms (float):  Numpy array containing VSOP87D periodic terms for heliocentric ecliptical latitude.
      radTerms (float):  Numpy array containing VSOP87D periodic terms for heliocentric distance.
    
    Returns:
      tuple (float,float,float):  Tuple containing (lon, lat, rad):
      
        - lon (float):  Heliocentric ecliptical longitude (rad).
        - lat (float):  Heliocentric ecliptical latitude (rad).
        - rad (float):  Heliocentric distance (km).
    
    References:
      - [Bretagnon & Francou, A&A 202, 309 (1988)](https://ui.adsabs.harvard.edu/abs/1988A%26A...202..309B).
    
    """
    
    Tjm = dt.jd2tjm(jde)  # Time since 2000 in Julian millennia
    
    lon=0.0; lat=0.0; rad=0.0
    
    for terms in lonTerms:
        cosTerm = terms[1] * m.cos(terms[2] + terms[3]*Tjm)
        lon += cosTerm * Tjm**terms[0]
        
    for terms in latTerms:
        cosTerm = terms[1] * m.cos(terms[2] + terms[3]*Tjm)
        lat += cosTerm * Tjm**terms[0]
        
    for terms in radTerms:
        cosTerm = terms[1] * m.cos(terms[2] + terms[3]*Tjm)
        rad += cosTerm * Tjm**terms[0]
        
    return lon % pi2, lat, rad



def hc2gc(le,be,re, lp,bp,rp):
    """
    Convert the heliocentric spherical coordinates of an object to geocentric spherical coordinates.
    
    Args:
      le (float):  Heliocentric ecliptical longitude of the Earth (rad).
      be (float):  Heliocentric ecliptical latitude of the Earth (rad).
      re (float):  Heliocentric ecliptical distance of the Earth (km).
    
      lp (float):  Heliocentric ecliptical longitude of the other object (rad).
      bp (float):  Heliocentric ecliptical latitude of the other object (rad).
      rp (float):  Heliocentric ecliptical distance of the other object (km).
    
    Returns:
      tuple (float,float,float):  Tuple containing (lon, lat, rad):
      
        - lon (float):  Geocentric ecliptical longitude (rad).
        - lat (float):  Geocentric ecliptical latitude (rad).
        - rad (float):  Geocentric distance (km).
    
    Note:
      - The heliocentric spherical coordinates of the Earth and the other object are first converted to
        rectangular coordinates.  Then the difference between the two sets is taken, yielding geocentric
        rectangular coordinates.  Finally, this difference is converted back to geocentric spherical
        coordinates.
      - When the coordinates of the Earth are replaced by those of a different object, the coordinates of the
        second object are computed, as seen from the centre of the first object.
      - This function is useful when using the VSOP87D files.
    
    """
    
    x = rp * m.cos(bp) * m.cos(lp)  -  re * m.cos(be) * m.cos(le)
    y = rp * m.cos(bp) * m.sin(lp)  -  re * m.cos(be) * m.sin(le)
    z = rp * m.sin(bp)              -  re * m.sin(be)
    
    # Convert geocentric rectangular to geocentric spherical coordinates:
    if (x==0) and (y==0) and (z==0):
        lon = 0.0
        lat = 0.0
        rad = 0.0
    else:
        x2 = x**2
        y2 = y**2
        
        lon = m.atan2(y, x)                # Longitude
        lat = m.atan2(z, m.sqrt(x2 + y2))  # Latitude
        rad = m.sqrt(x2 + y2 + z**2)       # Distance
       
    return lon,lat,rad



def xyz_hc2lbr_gc(xe,ye,ze, xp,yp,zp):
    """
    Convert the heliocentric rectangular coordinates of an object to geocentric spherical coordinates.
    
    Args:
      xe (float):  Heliocentric ecliptical x-coordinate of the Earth.
      ye (float):  Heliocentric ecliptical y-coordinate of the Earth.
      ze (float):  Heliocentric ecliptical z-coordinate of the Earth.
    
      xp (float):  Heliocentric ecliptical x-coordinate of the other object.
      yp (float):  Heliocentric ecliptical y-coordinate of the other object.
      zp (float):  Heliocentric ecliptical z-coordinate of the other object.
    
    Returns:
      tuple (float,float,float):  Tuple containing (lon, lat, rad):
      
        - lon (float):  Geocentric ecliptical longitude (rad).
        - lat (float):  Geocentric ecliptical latitude (rad).
        - rad (float):  Geocentric distance (km).
    
    Note:
      - The distance units of the arguments must all be the same.
      - When the coordinates of the Earth are replaced by those of a different object, the coordinates of the
        second object are computed, as seen from the centre of the first object.
      - This function is useful when using the VSOP87C files.
    
    """
    
    dx = xp - xe
    dy = yp - ye
    dz = zp - ze
    
    # Convert geocentric rectangular to spherical coordinates:
    if (dx==0) and (dy==0) and (dz==0):
        lon = 0.0
        lat = 0.0
        rad = 0.0
    else:
        dx2 = dx**2
        dy2 = dy**2
        
        lon = m.atan2(dy, dx)                 # Longitude
        lat = m.atan2(dz, m.sqrt(dx2 + dy2))  # Latitude
        rad = m.sqrt(dx2 + dy2 + dz**2)       # Distance
       
    return lon,lat,rad



def magnPlanet(pl, distPS, distPE, distSE):
    """Compute the apparent visual magnitude of a planet.
    
    Args:
      pl (int):   Planet ID (1-2, 4-9 for Mercury-Venus, Mars-Pluto).)
      distPS:     Heliocentric distance of the planet (AU).
      distPE:     Geocentric distance of the planet (AU).
      distSE:     Heliocentric distance of the Earth (AU).
    
    Returns:
      float:  Apparent visual magnitude of the planet.
    
    References:
      - [Explanatory Supplement to the Astronomical Almanac 3rd Ed, Table 10.6, p.413 +
        errata](https://aa.usno.navy.mil/publications/docs/exp_supp.php)

    """
    
    #               Mer    Ven1   Ven2   Mars   Jup    Sat    Ur     Nep    Pl
    a0 = np.array([-0.60, -4.47,  0.98, -1.52, -9.40, -8.88, -7.19, -6.87, -1.01])
    a1 = np.array([ 4.98,  1.03, -1.02,   1.6,   0.5,   4.4,   0.2,     0,     0]) * 1e-2
    a2 = np.array([-4.88,  0.57,     0,     0,     0,     0,     0,     0,     0]) * 1e-4
    a3 = np.array([ 3.02,  0.13,     0,     0,     0,     0,     0,     0,     0]) * 1e-6
    
    phAng = np.degrees( np.arccos( (distPS**2 + distPE**2 - distSE**2) / (2*distPS*distPE) ) )  # Phase angle (deg!)
    
    if (pl==2) and (phAng>163.6): pl = 3  # Venus 2
    pl = pl-1  # 1-9 -> 0-8
    
    mag = 5*np.log10(distPS*distPE) + a0[pl] + a1[pl]*phAng + a2[pl]*phAng**2 + a3[pl]*phAng**3
    
    return mag
    


def magnSatRing(JD, lon,lat):
    """Compute the apparent visual magnitude of Saturn's rings.
    
    Args:
      JD (float):   Julian Day (days).
      lon (float):  Geocentric ecliptical longitude of Saturn (rad).
      lat (float):  Geocentric ecliptical latitude of Saturn (rad).
    
    Returns:
      float:  magnSatRing: Apparent visual magnitude of the planet.
    
    References:
      - [Explanatory Supplement to the Astronomical Almanac 3rd Ed, Table 10.6, p.413 +
        errata](https://aa.usno.navy.mil/publications/docs/exp_supp.php)
    
    Note:
      - This is a simplified expression, which uses phi instead of Delta U.  The mean absolute deviation from
        the full expression: is 0.014m, the maximum deviation found: 0.041m in 10^5 trials, over the last 5000
        years.

    """
    
    tJC = dt.jd2tjc(JD)  # Time since 2000 in Julian centuries
    
    incl = 0.49                # Inclination of Saturn's rotation axis (rad)
    ascNod = 2.96 + 0.024*tJC  # Ascending node of Saturn's orbit (rad)
    
    sinB = np.sin(incl) * np.cos(lat) * np.sin(lon-ascNod)  -  np.cos(incl) * np.sin(lat)
    magnSatRing = -2.60*abs(sinB) + 1.25*(sinB)**2
    
    return magnSatRing
    



#  Copyright (c) 2019-2023  Marc van der Sluys - marc.vandersluys.nl
#   
#  This file is part of the HistAstro Python package,
#  see: http://astro.ru.nl/~sluys/HistAstro/
#   
#  This is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
#  
#  This software is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
#  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#  
#  You should have received a copy of the GNU General Public License along with this code.  If not, see
#  <http://www.gnu.org/licenses/>.


"""Coordinate transformations and related functions for HistAstro."""


# Modules:
import math as m
import numpy.core as np
from histastro.constants import pi2, r2d,as2r, earthRad,AU
import histastro.datetime as dt



def obliquity(jd):
    """
    Compute the obliquity of the ecliptic in radians from the JD(E).
    
    Arguments:
      jd (float):  Julian day (days).
    
    Returns:
      float:  eps: Obliquity of the ecliptic (rad).
    
    References:
      - Seidelman 1992, Eq. 3.222-1.
    
    """
    
    tJC = dt.jd2tjc(jd)  # Time in Julian centuries since J2000.0
    eps = 0.409092804 - 2.269655e-4*tJC - 2.86e-9*tJC**2 + 8.78967e-9*tJC**3  # Obliquity of the ecliptic (rad)
    
    return eps


def eq2ecl(ra,dec, eps):
    """
    Convert equatorial coordinates to ecliptical.

    Arguments:
      ra (float):   Right ascension (rad).
      dec (float):  Declination (rad).
      eps (float):  Obliquity of the ecliptic (rad).
    
    Returns:
      tuple (float,float): tuple containing (lon, lat):
    
        - lon (float):  Ecliptical longitude (rad).
        - lat (float):  Ecliptical latitude (rad).
    
    """
    
    lon = np.arctan2( np.sin(ra)  * m.cos(eps) + np.tan(dec) * m.sin(eps),  np.cos(ra) ) % pi2
    lat =  np.arcsin( np.sin(dec) * m.cos(eps) - np.cos(dec) * m.sin(eps) * np.sin(ra) )
    
    return lon,lat


def ecl2eq(lon,lat, eps):
    """Convert (geocentric) spherical ecliptical coordinates to spherical equatorial coordinates.
    
    Arguments:
      lon (float):  Ecliptical longitude (rad).
      lat (float):  Ecliptical latitude (rad).
      eps (float):  Obliquity of the ecliptic (rad).
    
    Returns:
      tuple (float,float): tuple containing (ra, dec):
    
        - ra (float):   Right ascension (rad).
        - dec (float):  Declination (rad).
    
    References:
      - [Explanatory Supplement to the Astronomical Almanac 3rd Ed,
        Eq.14.43](https://aa.usno.navy.mil/publications/docs/exp_supp.php)

    """
    
    ra  = np.arctan2( np.sin(lon) * np.cos(eps)  -  np.tan(lat) * np.sin(eps),  np.cos(lon) ) % pi2
    dec =  np.arcsin( np.sin(lat) * np.cos(eps)  +  np.cos(lat) * np.sin(eps) * np.sin(lon) )
    
    return ra,dec


def par2horiz(ha,dec, phi):
    """Convert parallactic coordinates to horizontal.

    Arguments:
      ha (float):   Hour angle (rad).
      dec (float):  Declination (rad).
      phi (float):  Geographical latitude (rad, N>0).
    
    Returns:
      tuple (float,float): tuple containing (az, alt):
    
        - az (float):   Azimuth (rad, S=0).
        - alt (float):  Altitude (rad).
    
    """
    
    az  = np.arctan2( np.sin(ha),   np.cos(ha) * m.sin(phi) - np.tan(dec) * m.cos(phi) ) % pi2
    alt = np.arcsin(  np.sin(dec) * m.sin(phi) + np.cos(ha) * np.cos(dec) * m.cos(phi) )
    
    return az,alt


def properMotion(startJD,targetJD, ra,dec, pma,pmd):
    """Compute the proper motion from startJD to targetJD for the given positions and proper motions.
    
    Arguments:
      startJD (float):   Julian day of the initial epoch (days).
      targetJD (float):  Julian day of the target epoch (days).
    
      ra (float):        Right ascension (numpy array, rad).
      dec (float):       Declination (numpy array, rad).
    
      pma (float):       Proper motion in right ascension (numpy array, rad/yr).
      pmd (float):       Proper motion in declination (numpy array, rad/yr).

    Returns:
      tuple (float,float): tuple containing (raTarget, decTarget):
    
        - raTarget (float):   Right ascension for the target epoch (rad).
        - decTarget (float):  Declination for the target epoch (rad).

    """
    
    dtYr   = (targetJD - startJD)/365.25
    raTarget  = ra  + pma*dtYr / np.cos(dec)
    decTarget = dec + pmd*dtYr
    
    return raTarget,decTarget


def precessHip(jd, ra,dec):
    """Compute precession in equatorial coordinates from the Hipparcos equinox (J2000) to that of the specified JD.
    
    Arguments:
      jd (float):   Julian day (days).
      ra (float):   Right ascension (rad).
      dec (float):  Declination (rad).
    
    Returns:
      tuple (float,float): tuple containing (raTarget, decTarget):
    
        - raNew (float):   Right ascension for the target equinox (rad).
        - decNew (float):  Declination for the target equinox (rad).
    
    """
    
    tJC  = dt.jd2tjc(jd)  # Time in Julian centuries since J2000.0
    tJC2 = tJC**2
    tJC3 = tJC*tJC2
    
    zeta  = (2306.2181*tJC + 0.30188*tJC2 + 0.017998*tJC3)*as2r
    z     = (2306.2181*tJC + 1.09468*tJC2 + 0.018203*tJC3)*as2r
    theta = (2004.3109*tJC - 0.42665*tJC2 - 0.041833*tJC3)*as2r
    
    raNew  = (np.arctan2( np.sin(ra + zeta) * np.cos(dec),  np.cos(ra + zeta) * m.cos(theta) * np.cos(dec) - m.sin(theta) * np.sin(dec) ) + z) % pi2
    decNew = np.arcsin( np.cos(ra + zeta) * m.sin(theta) * np.cos(dec)  +  m.cos(theta) * np.sin(dec) )
    
    return raNew,decNew


def geoc2topoc_ecl(gcLon,gcLat, gcDist,gcRad, eps,lst, obsLat,obsEle=0, debug=False):
    """Convert spherical ecliptical coordinates from the geocentric to the topocentric system.
    
    Arguments:
      gcLon (float):   Geocentric ecliptic longitude (rad).
      gcLat (float):   Geocentric ecliptic latitude (rad).
      gcDist (float):  Geocentric distance (AU).
      gcRad (float):   Geocentric semi-diameter (rad).
      
      eps (float):     Obliquity of the ecliptic (rad).
      lst (float):     Local sidereal time (rad).
      
      obsLat (float):  Geographical latitude of the observer (rad).
      obsEle (float):  Altitude/elevation of the observer above sea level (metres, optional, default value = 0).
      
      debug (float):   Print debug output (True/False, optional, default value = True).
    
    Returns:
      tuple (float,float,float): tuple containing (tcLon, tcLat, tcRad):
    
        - tcLon (float):  Topocentric ecliptic longitude (rad).
        - tcLat (float):  Topocentric ecliptic latitude (rad).
        - tcRad (float):  Topocentric semi-diameter (rad).
    
    """
    
    # Meeus, Ch.11, p.82:
    Req = earthRad*1000      # Equatorial radius of the Earth in metres (same units as the elevation)
    #                        (http://earth-info.nga.mil/GandG/publications/tr8350.2/wgs84fin.pdf)
    RpolEq = 0.996647189335  # Rpol/Req = 1-f: flattening of the Earth - WGS84 ellipsoid 
    
    u  = m.atan(RpolEq*m.tan(obsLat))
    RsinPhi = RpolEq*m.sin(u) + obsEle/Req * m.sin(obsLat)
    RcosPhi = m.cos(u)        + obsEle/Req * m.cos(obsLat)
    
    sinHp = m.sin(earthRad/AU)/(gcDist/AU)  # Sine of the horizontal parallax, Meeus, Eq. 40.1
    
    # Meeus, Ch.40, p.282:
    N  = m.cos(gcLon)*m.cos(gcLat) - RcosPhi*sinHp*m.cos(lst)
    
    tcLon = m.atan2( m.sin(gcLon)*m.cos(gcLat) - sinHp*(RsinPhi*m.sin(eps) + RcosPhi*m.cos(eps)*m.sin(lst)), N ) % pi2  # Topocentric longitude
    tcLat = m.atan((m.cos(tcLon)*(m.sin(gcLat) - sinHp*(RsinPhi*m.cos(eps) - RcosPhi*m.sin(eps)*m.sin(lst))))/N)         # Topocentric latitude
    tcRad = m.asin(m.cos(tcLon)*m.cos(tcLat)*m.sin(gcRad)/N)                                                   # Topocentric semi-diameter
    
    # print(gcDist, gcDist*gcRad/tcRad)
    
    if debug:
        print()
        print('geoc2topoc_ecl():')
        print('%10s  %25s  %25s' % ('', 'rad/km/...','deg'))
        print()
        print('%10s  %25.15f' % ('Req: ', Req) )
        print('%10s  %25.15f' % ('RpolEq: ', RpolEq) )
        print()
        print('%10s  %25.15f  %25.15f' % ('u: ', u, u*r2d) )
        print('%10s  %25.15f' % ('RsinPhi: ', RsinPhi) )
        print('%10s  %25.15f' % ('RcosPhi: ', RcosPhi) )
        print()
        print('%10s  %25.15f' % ('sinHp: ', sinHp) )
        print('%10s  %25.15f  %25.15f' % ('N: ', N, N*r2d) )
        print()
        print('%10s  %25.15f  %25.15f' % ('tcLon: ', tcLon, tcLon*r2d) )
        print('%10s  %25.15f  %25.15f' % ('tcLat: ', tcLat, tcLat*r2d) )
        print('%10s  %25.15f  %25.15f' % ('tcRad: ', tcRad, tcRad*r2d) )
        
    return tcLon,tcLat,tcRad






#  Copyright (c) 2019-2023  Marc van der Sluys - marc.vandersluys.nl
#   
#  This file is part of the HistAstro Python package,
#  see: http://astro.ru.nl/~sluys/HistAstro/
#   
#  This is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
#  
#  This software is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
#  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#  
#  You should have received a copy of the GNU General Public License along with this code.  If not, see
#  <http://www.gnu.org/licenses/>.


"""Moon functions for HistAstro."""


import math as m
import numpy as np
from histastro.constants import pi2,r2d,jd2000,moonRad  # ,d2r


def readData(inFile='data/moonposMeeus.csv'):
    """
    Return the periodic terms for the ELP82B theory from moonposMeeus.csv.

      - Two arrays aand return them in two arrays: one for longitude and distance, and one for latitude.

    Args: 
      inFile (str): Name of the input file, including (relative or absolute) path.  Optional, default
      value = 'data/moonposMeeus.csv'.

    Returns:
      tuple (float,float):  Tuple containing (lrTerms, bTerms):

        - lrTerms (float): Numpy array containing six columns with periodic terms for longitude and distance.
          The first four columns contain arguments for both variables, the last two the coefficients for
          longitude and distance:
    
          - 1 (int): Multiplication factor for the mean elongation of the Moon.
          - 2 (int): Multiplication factor for the mean anomaly of the Sun.
          - 3 (int): Multiplication factor for the mean anomaly of the Moon.
          - 4 (int): Multiplication factor for the Moon's argument of latitude.
          - 5 Coefficient of the sine of the argument, for the longitude (rad).
          - 6 Coefficient of the cosine of the argument, for the distance (km).
    
        - bTerms (float): Numpy array containing five columns periodic terms for latitude.  The first four
          columns contain arguments, the last contains the coefficients for latitude:
    
          - 1 (int): Multiplication factor for the mean elongation of the Moon.
          - 2 (int): Multiplication factor for the mean anomaly of the Sun.
          - 3 (int): Multiplication factor for the mean anomaly of the Moon.
          - 4 (int): Multiplication factor for the Moon's argument of latitude.
          - 5 Coefficient of the sine of the argument, for the latitude (rad).
    
    References:
      - [Jean Meeus: Astronomical Algorithms, 2nd Ed. (1998)](https://www.willbell.com/math/MC1.HTM), Ch.47.

    """
    
    lrTerms = np.genfromtxt(inFile, delimiter=',', skip_header=1,  max_rows=60)  # Longitude and radius (6 columns: 4 args, 2 coefs)
    bTerms  = np.genfromtxt(inFile, delimiter=',', skip_header=61, max_rows=60)  # Latitude (5 columns: 4 args, 1 coef)
    
    return lrTerms,bTerms


def compute_lbr(jde, lrTerms,bTerms, debug=False):
    """
    Compute the geocentric ecliptic coordinates of the Moon for the equinox of date for the given JDE.
    
    Args:
      jde (float):      Julian Day in dynamical time, i.e., corrected for Delta T (days).
      lrTerms (float):  Numpy array with ELP82 periodic terms for longitude and distance.
      bTerms (float):   Numpy array with ELP82 periodic terms for latitude.
      debug (bool):     Produce debug output.  Optional, default value = False.
    
    Returns:
      tuple (float,float,float,float):  Tuple containing (lon, lat, dist, diam):
    
        - lon (float):   Geocentric ecliptic longitude of the Moon (rad).
        - lat (float):   Geocentric ecliptic longitude of the Moon (rad).
        - dist (float):  Geocentric distance of the Moon (km).
        - diam (float):  Geocentric apparent diameter of the Moon (rad).
    
    Notes:
      - A reduced version of the ELP82 theory is used.
      - The necessary ELP82 terms can be read from file using the function readData().
    
    References:
      - [Chapront-Touze & Chapront, A&A 190, 342 (1988)](https://ui.adsabs.harvard.edu/abs/1988A%26A...190..342C).
    
    """
    
    tjc   = (jde-jd2000)/36525  # Julian Centuries after 2000.0 in dynamical time
    tjc2  = tjc**2
    tjc3  = tjc*tjc2
    tjc4  = tjc2**2
    
    # Moon's mean longitude, Meeus p.338:
    lm = (3.8103408236 + 8399.7091116339958*tjc - 2.755176757e-5*tjc2 + 3.239043e-8*tjc3 - 2.6771e-10*tjc4) % pi2
    
    # Delauney arguments [d, ms, mm, f] (Meeus p.338) = [D, l', l, F] in Simon et al. 1994, Sect. 3.5:
    d  = (5.1984665298  + 7771.377144834*tjc - 3.2845e-5*tjc2  + 3.197347e-8*tjc3    - 1.5436512e-10*tjc4) % pi2  # Moon's mean elongation
    ms = (6.240060127   + 628.301955167*tjc  - 2.681e-6*tjc2   + 7.1267017e-10*tjc3)                       % pi2  # Sun's mean anomaly
    mm = (2.355555637   + 8328.691424759*tjc + 1.52566e-4*tjc2 + 2.5041e-7*tjc3      - 1.18633e-9*tjc4)    % pi2  # Moon's mean anomaly
    f  = (1.627905158   + 8433.466158061*tjc - 6.3773e-5*tjc2  - 4.94988e-9*tjc3     + 2.02167e-11*tjc4)   % pi2  # Moon's argument of latitute
    DelArgs = [d,ms,mm,f]  # Delauney arguments
    
    e  = 1 - 0.002516*tjc - 0.0000074*tjc2
    
    # Compute ELP82 terms:
    lon=0; lat=0; dist=0
    for iLine in range(60):  # Data lines
        argl = 0
        argb = 0
        for iArg in range(4):  # Integer arguments
            argl += lrTerms[iLine,iArg] * DelArgs[iArg]
            argb +=  bTerms[iLine,iArg] * DelArgs[iArg]
        
        lon  += m.sin(argl) * lrTerms[iLine,4] * e**(abs(lrTerms[iLine,1]))
        lat  += m.sin(argb) *  bTerms[iLine,4] * e**(abs(bTerms[iLine,1]))
        dist += m.cos(argl) * lrTerms[iLine,5] * e**(abs(lrTerms[iLine,1]))
        
    # Save to print intermediate results:
    lon1  = lon
    lat1  = lat
    dist1 = dist
    
    
    # Perturbations by other planets, and flattening of the Earth:
    # Meeus, p.338:
    a1 = (2.090032  +     2.301199 * tjc) % pi2  # Influence of Venus
    a2 = (0.926595  + 8364.7398477 * tjc) % pi2  # Influence of Jupiter
    a3 = (5.4707345 + 8399.6847253 * tjc) % pi2
    
    # Meeus, p.342:
    # dlon = (3958*m.sin(a1) + 1962*m.sin(lm-f) + 318*m.sin(a2)) * d2r * 1e-6
    # dlat = (-2235*m.sin(lm) + 382*m.sin(a3) + 175*m.sin(a1-f) + 175*m.sin(a1+f) + 127*m.sin(lm-mm) - 115*m.sin(lm+mm)) * d2r * 1e-6
    dlon =  6.908e-5*m.sin(a1) + 3.4243e-5*m.sin(lm-f) + 5.55e-6*m.sin(a2)
    dlat = -3.9008e-5*m.sin(lm) + 6.667e-6*m.sin(a3) + 3.0543e-6*m.sin(a1-f) + 3.0543e-6*m.sin(a1+f) + 2.2166e-6*m.sin(lm-mm) - 2.007e-6*m.sin(lm+mm)
    
    
    # Compute nutation in longitude:
    # omg  = 2.18243858558 - 33.7570459367*tjc + 3.6142278e-5*tjc2 + 3.87850944888e-8*tjc3   # Moon's mean lon. of asc.node, Meeus p.144
    # ls   = 4.89506386655 + 62.84528862*tjc                                                 # Mean long. Sun, Meeus p.144
    # dpsi = -8.338795e-5*m.sin(omg) - 6.39954e-6*m.sin(2*ls) - 1.115e-6*m.sin(2*lm) + 1.018e-6*m.sin(2*omg)
    dpsi = 0
    
    # Add mean values:
    lon  = (lon + dlon + lm + dpsi) % pi2
    lat  = lat + dlat
    dist = dist + 385000.56  # in km
    
    diam = 2*m.atan(moonRad/dist)
    
    if debug:
        print()
        print('compute_lbr():')
        print('%10s  %25s  %25s' % ('', 'rad/km/...','deg'))
        print()
        print('%10s  %25.15f' % ('jde:  ', jde) )
        print('%10s  %25.15f' % ('tjc:  ', tjc) )
        print()
        print('%10s  %25.15f  %25.15f' % ('lm:  ', lm, lm*r2d) )
        print()
        print('%10s  %25.15f  %25.15f' % ('d:   ', d, d*r2d) )
        print('%10s  %25.15f  %25.15f' % ('ms:  ', ms, ms*r2d) )
        print('%10s  %25.15f  %25.15f' % ('mm:  ', mm, mm*r2d) )
        print('%10s  %25.15f  %25.15f' % ('f:   ', f, f*r2d) )
        print()
        print('%10s  %25.15f' % ('e:   ', e) )
        print()
        print('%10s  %25.15f  %25.15f' % ('a1:  ', a1, a1*r2d) )
        print('%10s  %25.15f  %25.15f' % ('a2:  ', a2, a2*r2d) )
        print('%10s  %25.15f  %25.15f' % ('a3:  ', a3, a3*r2d) )
        print()
        print("ELP82 sums:")
        print('%10s  %25.15f  %25.15f' % ('lon: ', lon1, lon1*r2d) )
        print('%10s  %25.15f  %25.15f' % ('lat: ', lat1, lat1*r2d) )
        print('%10s  %25.15f' % ('dist:', dist1) )
        print()
        print('%10s  %25.15f  %25.15f' % ('dlon: ', dlon, dlon*r2d) )
        print('%10s  %25.15f  %25.15f' % ('dlat: ', dlat, dlat*r2d) )
        print()
        print('%10s  %25.15f  %25.15f' % ('lon:  ', lon, lon*r2d) )
        print('%10s  %25.15f  %25.15f' % ('lat:  ', lat, lat*r2d) )
        print('%10s  %25.15f' % ('dist:  ', dist) )
        print('%10s  %25.15f  %25.15f' % ('diam:  ', diam, diam*r2d) )
        print()
        #    print('%10s  %25.15f  %25.15f' % ('x:  ', x*r2d) )
    
    return lon,lat,dist,diam





#  Copyright (c) 2019-2023  Marc van der Sluys - marc.vandersluys.nl
#   
#  This file is part of the HistAstro Python package,
#  see: http://astro.ru.nl/~sluys/HistAstro/
#   
#  This is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
#  
#  This software is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
#  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#  
#  You should have received a copy of the GNU General Public License along with this code.  If not, see
#  <http://www.gnu.org/licenses/>.


"""Definition of constants for HistAstro."""

# Modules:
import math as m

pi   = m.pi;           """pi"""
pi2  = 2*pi;           """2 pi"""
pio2 = pi/2;           """pi/2"""

r2d  = m.degrees(1);   """Radians to degrees"""
d2r  = m.radians(1);   """Degrees to radians"""

h2r   = d2r*15;        """Hours to radians"""
as2r  = d2r/3.6e3;     """Arcseconds to radians"""
mas2r = as2r/1000.0;   """Milliarcseconds to radians"""

jd1820 = 2385801;      """JD in 1820  (for Delta-T fit)"""
jd1900 = 2415021;      """JD in 1900"""
jd2000 = 2451545;      """JD in 2000.0"""

earthRad = 6378.1366;  """Earth radius in km"""
moonRad = 1737.5;      """Moon radius in km"""

AU = 1.495978707e8;    """Astronomical unit in km"""




#  Copyright (c) 2019-2023  Marc van der Sluys - marc.vandersluys.nl
#   
#  This file is part of the HistAstro Python package,
#  see: http://astro.ru.nl/~sluys/HistAstro/
#   
#  This is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
#  
#  This software is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
#  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#  
#  You should have received a copy of the GNU General Public License along with this code.  If not, see
#  <http://www.gnu.org/licenses/>.


"""HistAstro module

HistAstro is a Python package for basic historical-astronomy calculations of Sun, Moon and planets, which can
be used under the conditions of the GPLv3 licence.  These pages contain the API documentation.  For more
information on the Python package, licence, source code and data files, see the [HistAstro
homepage](http://astro.ru.nl/~sluys/HistAstro/).

"""


name = "histastro"

